﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.Collections.Specialized;
using System.Configuration;
using System.Linq;
using System.Web;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.Configuration;
using VeteransAffairs.Registries.Business;
using VeteransAffairs.Registries.BusinessManager.TBI;

/// <summary>
/// Summary description for Helpers
/// </summary>
public static class Helpers
{
    private static string[] _countries = new string[] { 
         "Afghanistan", "Albania", "Algeria", "American Samoa", "Andorra", 
         "Angola", "Anguilla", "Antarctica", "Antigua And Barbuda", "Argentina", 
         "Armenia", "Aruba", "Australia", "Austria", "Azerbaijan",
		   "Bahamas", "Bahrain", "Bangladesh", "Barbados", "Belarus",
		   "Belgium", "Belize", "Benin", "Bermuda", "Bhutan",
		   "Bolivia", "Bosnia Hercegovina", "Botswana", "Bouvet Island", "Brazil",
		   "Brunei Darussalam", "Bulgaria", "Burkina Faso", "Burundi", "Byelorussian SSR",
		   "Cambodia", "Cameroon", "Canada", "Cape Verde", "Cayman Islands",
		   "Central African Republic", "Chad", "Chile", "China", "Christmas Island",
		    "Colombia", "Comoros", "Congo", "Cook Islands",
		   "Costa Rica", "Cote D'Ivoire", "Croatia", "Cuba", "Cyprus",
		   "Czech Republic", "Czechoslovakia", "Denmark", "Djibouti", "Dominica",
		   "Dominican Republic", "East Timor", "Ecuador", "Egypt", "El Salvador",
		   "England", "Equatorial Guinea", "Eritrea", "Estonia", "Ethiopia",
		   "Falkland Islands", "Faroe Islands", "Fiji", "Finland", "France",
		   "Gabon", "Gambia", "Georgia", "Germany", "Ghana",
		   "Gibraltar", "Great Britain", "Greece", "Greenland", "Grenada",
		   "Guadeloupe", "Guam", "Guatemela", "Guernsey", "Guiana",
		   "Guinea", "Guinea-Bissau", "Guyana", "Haiti", "Heard Islands",
		   "Honduras", "Hong Kong", "Hungary", "Iceland", "India",
		   "Indonesia", "Iran", "Iraq", "Ireland", "Isle Of Man",
		   "Israel", "Italy", "Jamaica", "Japan", "Jersey",
		   "Jordan", "Kazakhstan", "Kenya", "Kiribati", "Korea, South",
		   "Korea, North", "Kuwait", "Kyrgyzstan", "Lao People's Dem. Rep.", "Latvia",
		   "Lebanon", "Lesotho", "Liberia", "Libya", "Liechtenstein",
		   "Lithuania", "Luxembourg", "Macau", "Macedonia", "Madagascar",
		   "Malawi", "Malaysia", "Maldives", "Mali", "Malta",
		   "Mariana Islands", "Marshall Islands", "Martinique", "Mauritania", "Mauritius",
		   "Mayotte", "Mexico", "Micronesia", "Moldova", "Monaco",
		   "Mongolia", "Montserrat", "Morocco", "Mozambique", "Myanmar",
		   "Namibia", "Nauru", "Nepal", "Netherlands", "Netherlands Antilles",
		   "Neutral Zone", "New Caledonia", "New Zealand", "Nicaragua", "Niger",
		   "Nigeria", "Niue", "Norfolk Island", "Northern Ireland", "Norway",
		   "Oman", "Pakistan", "Palau", "Panama", "Papua New Guinea",
		   "Paraguay", "Peru", "Philippines", "Pitcairn", "Poland",
		   "Polynesia", "Portugal", "Puerto Rico", "Qatar", "Reunion",
		   "Romania", "Russian Federation", "Rwanda", "Saint Helena", "Saint Kitts",
		   "Saint Lucia", "Saint Pierre", "Saint Vincent", "Samoa", "San Marino",
		   "Sao Tome and Principe", "Saudi Arabia", "Scotland", "Senegal", "Seychelles",
		   "Sierra Leone", "Singapore", "Slovakia", "Slovenia", "Solomon Islands",
		   "Somalia", "South Africa", "South Georgia", "Spain", "Sri Lanka",
		   "Sudan", "Suriname", "Svalbard", "Swaziland", "Sweden",
		   "Switzerland", "Syrian Arab Republic", "Taiwan", "Tajikista", "Tanzania",
		   "Thailand", "Togo", "Tokelau", "Tonga", "Trinidad and Tobago",
		   "Tunisia", "Turkey", "Turkmenistan", "Turks and Caicos Islands", "Tuvalu",
		   "Uganda", "Ukraine", "United Arab Emirates", "United Kingdom", "United States",
		   "Uruguay", "Uzbekistan", "Vanuatu", "Vatican City State", "Venezuela",
		   "Vietnam", "Virgin Islands", "Wales", "Western Sahara", "Yemen",
		   "Yugoslavia", "Zaire", "Zambia", "Zimbabwe"};

    /// <summary>
    /// Returns an array with all countries
    /// </summary>     
    public static StringCollection GetCountries()
    {
        StringCollection countries = new StringCollection();
        countries.AddRange(_countries);
        return countries;
    }
    /// <summary>
    /// Returns sorted list of all countries
    /// </summary>
    /// <param name="insertEmpty"></param>
    /// <returns></returns>
    public static SortedList GetCountries(bool insertEmpty)
    {
        SortedList countries = new SortedList();
        if (insertEmpty)
            countries.Add("", "Please select one...");
        foreach (String country in _countries)
            countries.Add(country, country);
        return countries;
    }
    /// <summary>
    /// highlights controls on selection 
    /// </summary>
    /// <param name="container"></param>
    /// <param name="className"></param>
    public static void SetInputControlsHighlight(Control container, string className)
    {
        foreach (Control ctl in container.Controls)
        {
            if ( ctl is TextBox || ctl is DropDownList ||
                ctl is ListBox || ctl is CheckBox || ctl is RadioButton ||
                ctl is RadioButtonList || ctl is CheckBoxList )
            {
                WebControl wctl = ctl as WebControl;
                wctl.Attributes.Add("onfocus", string.Format("this.className = '{0}';", className));
                wctl.Attributes.Add("onblur", "this.className = '';");
            }
            else
            {
                if (ctl.Controls.Count > 0)
                    SetInputControlsHighlight(ctl, className);
            }
        }
    }

    public static Boolean isInteger(string valueToCheck)
    {
        long _tempInt; 
		
		try
			{
				_tempInt = Convert.ToInt64( valueToCheck );
                return true;
			}
			catch
			{
				return false;
			}

    }

    public static Boolean isString(string valueToCheck)
    {
        string _tempString; 
		
		try
			{
                _tempString = valueToCheck.ToString();
                return true;
			}
			catch
			{
				return false;
			}

    }

    public static string GetConfigEntry(string key)
    {
        string Val = string.Empty ;   
        if (WebConfigurationManager.AppSettings.Count > 0)
                Val = WebConfigurationManager.AppSettings.Get(key);

        return Val;
    }

    public static string GetDBConfigEntry(string key)
    {
        RegistriesLinqDataContext dc = new RegistriesLinqDataContext();

        var setting = (from s in dc.CRS_SETTINGSs
                       where s.NAME == key
                       select s.VALUE).SingleOrDefault();

        if (setting == null)
            setting = "";

        return setting;
    }

    /// <summary>
    /// getCurrentServiceToUse: pulls from settings.  If null, returns MDWS, else returns either MDWS or VIA
    /// </summary>
    /// <returns></returns>
    public static string getCurrentServiceToUse()
    {
        string resultStr = "MDWS";

        try
        {
            var setting = GetDBConfigEntry("TBIServiceToUse");

            if (setting != null)
            {
                switch (setting.ToString().ToLower())
                {
                    case "mdws":
                        resultStr = "MDWS";
                        break;
                    case "via":
                        resultStr = "VIA";
                        break;
                    default:
                        resultStr = "MDWS";
                        break;
                }
            }
        }
        catch (Exception e)
        {           
            Logging.WriteLogToFile("Error in GetCurrentServiceToUse(): " + e.Message);
        }

        return resultStr;
    }

    public static int getIntFromString(string inString)
    {
        int returnInt = 0;

        int tempInt;
        bool result = int.TryParse(inString, out tempInt);
        if (result)
        {
            returnInt = tempInt;
        }

        return returnInt;
    }

    public static bool IsSessionVariableNullOrEmpty(object sessionVariable)
    {
        return string.IsNullOrEmpty(GetStringFromSessionVariable(sessionVariable));
    }

    public static string GetStringFromSessionVariable(object sessionVariable)
    {
        string result = string.Empty;

        if (sessionVariable != null)
        {
           result = sessionVariable.ToString();
        }

        return result;
    }
    
    public static int? getNullableIntFromString(string inString)
    {
        int? returnInt = null;

        int tempInt;
        bool result = int.TryParse(inString, out tempInt);
        if (result)
        {
            returnInt = tempInt;
        }

        return returnInt;
    }

    public static string getStringFromNullableInt(int? inInt)
    {
        string returnString = "";

        if (inInt.HasValue)
        {
            returnString = inInt.ToString();
        }

        return returnString;
    }

    public static DateTime? getNullableDateFromString(string inString)
    {
        DateTime? returnDate = null;

        DateTime tempDate;
        bool result = DateTime.TryParse(inString, out tempDate);
        if (result)
        {
            returnDate = tempDate;
        }

        return returnDate;
    }

    public static Char? getNullableCharFromString(string inString)
    {
        Char? returnChar = null;

        Char tempChar;
        bool result = Char.TryParse(inString, out tempChar);
        if (result)
        {
            returnChar = tempChar;
        }

        return returnChar;
    }

    public static string getNullableStringFromString(string inString)
    {
        string returnString = null;

        if (!String.IsNullOrEmpty(inString))
        {
            returnString = inString;
        }

        return returnString;
    }

    public static string getShortDateStringFromNullableDate(DateTime? inDate)
    {
        string returnString = "";

        if (inDate.HasValue)
        {
            returnString = Convert.ToDateTime(inDate).ToShortDateString();
            
        }

        return returnString;
    }
    
    public static Exception HandleTBIException(Exception exception, string screenName, string eventId, string functionname)
    {
        WriteLogToFile(exception, screenName, eventId, functionname);
        return exception;
    }

    private static void WriteLogToFile(Exception ex, string _screenName, string _eventId, string _functionname)
    {
        string path = GetLogLocation();
        string flatmessage = "TBI INSTRUMENT ERROR:: " + " :::SCREENNAME= " + _screenName + ":::Function=" + _functionname;
        
        if (ex != null)
        {
            flatmessage += " ::: EXCEPTION= " + ex.ToString();            
        }              
        Logging.WriteLogToFile(flatmessage, true);    
    } 

    private static string GetLogLocation()
    {
        return ConfigurationManager.AppSettings["LogsPath"].ToString() + @"\" + DateTime.Today.Year + @"\" + DateTime.Today.Month + @"\" + DateTime.Today.Day + ".log";
    }


    public static TBIServiceTO BuildTBIServiceTO(string _inSiteCode, string _inDUZ, string _inDFN, string _inServerIP, string _inServerPort, int _inTBIEvaluationNoteId, int _inTBIFollowUpNoteId)
    {
        TBIServiceTO result = new TBIServiceTO();
        result.siteCode = _inSiteCode;
        result.DUZ = _inDUZ;
        result.DFN = _inDFN;
        result.CprsSentTBIEvaluationNoteId = _inTBIEvaluationNoteId;
        result.CprsSentTBIFollowUpNoteId = _inTBIFollowUpNoteId;
        result.ServerIP = _inServerIP;
        result.ServerPort = _inServerPort;
        return result;
    }

    public static void ExtractCPRSVariables(Page page, out string _inSiteCode, out string _inDUZ, out string _inDFN, out string _inServerIP, out string _inServerPort, out int _inTBIEvaluationNoteId, out int _inTBIFollowUpNoteId)
    {
        ExtractCPRSVariables(page.Session, page.Response, out _inSiteCode, out _inDUZ, out _inDFN, out _inServerIP, out _inServerPort, out _inTBIEvaluationNoteId, out _inTBIFollowUpNoteId);
    }

    public static void ExtractCPRSVariables(System.Web.SessionState.HttpSessionState session, HttpResponse response, out string _inSiteCode, out string _inDUZ, out string _inDFN, out string _inServerIP, out string _inServerPort, out int _inTBIEvaluationNoteId, out int _inTBIFollowUpNoteId)
    {
        _inSiteCode = Helpers.GetStringFromSessionVariable(session["SiteCode"]);
        _inDUZ = Helpers.GetStringFromSessionVariable(session["DUZ"]);
        _inDFN = Helpers.GetStringFromSessionVariable(session["DFN"]);
        _inServerIP = Helpers.GetStringFromSessionVariable(session["ServerIP"]);
        _inServerPort = Helpers.GetStringFromSessionVariable(session["ServerPort"]);

        int.TryParse(Helpers.GetStringFromSessionVariable(session["TBIEvaluationNoteId"]), out _inTBIEvaluationNoteId);
        int.TryParse(Helpers.GetStringFromSessionVariable(session["TBIFollowUpNoteId"]), out _inTBIFollowUpNoteId);

        if (string.IsNullOrEmpty(_inDUZ)
            || string.IsNullOrEmpty(_inDFN)
            || string.IsNullOrEmpty(_inSiteCode)
            || string.IsNullOrEmpty(_inServerIP)
            || string.IsNullOrEmpty(_inServerPort))
        {
            string sError = "PatientInstrumentsMain.Page_Load(): WarningRedirect: null or missing query string values: ";
            if (Helpers.IsSessionVariableNullOrEmpty(session["DUZ"]))
                sError += "DUZ;";
            if (Helpers.IsSessionVariableNullOrEmpty(session["DFN"]))
                sError += "DFN;";
            if (Helpers.IsSessionVariableNullOrEmpty(session["SiteCode"]))
                sError += "SiteCode;";
            if (Helpers.IsSessionVariableNullOrEmpty(session["ServerIP"]))
                sError += "ServerIP";
            if (Helpers.IsSessionVariableNullOrEmpty(session["ServerPort"]))
                sError += "ServerPort";

            Logging.WriteLogToFile("PatientInstruments.Page_Load(): WarningRedirect: " + sError);
            //if any value is not passed in then the page was called incorrectly, display error message to user
            response.Redirect("Warning.aspx?err=7", false);
        }
    }

    #region TBI Instruments

    //Shared method to save radio buttons. Before using this make sure the code being replaced is EXACTLY the same.
    public static void SaveSingleSelect(SURVEY survey, RadioButtonList radioButtonList, string questionNumber)
    {
        if (radioButtonList.SelectedIndex > -1)
        {
            List<STD_QUESTION_CHOICE> availableChoices = VeteransAffairs.Registries.QuestionChoiceRetriever.GetQuestionChoices(questionNumber);
            SURVEY_RESULT currentResult = survey.SURVEY_RESULTs.FirstOrDefault(item => availableChoices.FirstOrDefault(choice => choice.STD_QUESTION_CHOICE_ID == item.STD_QUESTION_CHOICE_ID) != null);
            int selectedValue = Convert.ToInt32(radioButtonList.SelectedValue);
            if (null != currentResult)
            {
                if (currentResult.STD_QUESTION_CHOICE_ID != selectedValue)
                {
                    currentResult.SetAsUpdateOnSubmit();
                    currentResult.STD_QUESTION_CHOICE_ID = selectedValue;
                }
                else
                {
                    currentResult.SetAsNoChangeOnSubmit();
                }
            }
            else
            {
                SURVEY_RESULT result = new SURVEY_RESULT();
                survey.SURVEY_RESULTs.Add(result);
                result.STD_QUESTION_CHOICE_ID = selectedValue;
            }
        }
    }

    public static int[] GetSelectedIndexes(CheckBoxList chkList)
    {
        List<int> selectedIndexes = new List<int>();
        foreach (ListItem item in chkList.Items)
        {
            if (item.Selected)
            {
                selectedIndexes.Add(chkList.Items.IndexOf(item));
            }
        }
        return selectedIndexes.ToArray();
    }

    public static bool SetupPageFromCache(Page page, string cacheName)
    {
        Dictionary<string, string> cache;
        return SetupPageFromCache(page, cacheName, out cache);
    }

    public static bool SetupPageFromCache(Page page, string cacheName, out Dictionary<string, string> cache)
    {
        cache = page.Session[cacheName] as Dictionary<string, string>;
        if (null == cache)
            return false;
        SetupControlFromCache(page, cache);
        return true;
    }

    private static void SetupControlFromCache(Control control, Dictionary<string, string> cache)
    {
        if (null == control)
            return;
        string key = control.ID;
        string value = null;
        try
        {
            if (null != key)
            {
                if (cache.TryGetValue(key, out value) && value != null)
                {
                    if (control is TextBox)
                        (control as TextBox).Text = value;
                    else if (control is CheckBox)
                    {
                        (control as CheckBox).Checked = bool.Parse(value);
                    }
                    else if (control is CheckBoxList)
                    {
                        string[] indexes = value.Split(new char[] { ',' }, StringSplitOptions.RemoveEmptyEntries);
                        if (null != indexes)
                        {
                            for (int i = 0; i < (control as CheckBoxList).Items.Count; i++)
                            {
                                (control as CheckBoxList).Items[i].Selected = indexes.Contains(i.ToString());
                            }
                        }
                    }
                    else if (control is DropDownList)
                        (control as DropDownList).SelectedIndex = int.Parse(value);
                    else if (control is RadioButtonList)
                        (control as RadioButtonList).SelectedIndex = int.Parse(value);
                }
            }
        }
        catch (Exception x)
        {
            Logging.WriteLogToFile(string.Format("Error: key={0}, value={1}\r\n", key, value) + x.ToString());
        }
        if (!(control is Repeater))
        {
            foreach (Control child in control.Controls)
                SetupControlFromCache(child, cache);
        }
    }

    public static Dictionary<string, string> SavePageToCache(Page page, string cacheName)
    {
        Dictionary<string, string> cache = new Dictionary<string, string>();
        CacheControlValue(page, cache);
        page.Session[cacheName] = cache;
        return cache;
    }

    private static void CacheControlValue(Control control, Dictionary<string, string> cache)
    {
        if (null == control)
            return;
        if (control is TextBox)
            cache[control.ID] = (control as TextBox).Text;
        else if (control is CheckBoxList)
            cache[control.ID] = string.Join(",", Helpers.GetSelectedIndexes(control as CheckBoxList).Select(i => i.ToString()).ToArray());
        else if (control is RadioButtonList)
            cache[control.ID] = (control as RadioButtonList).SelectedIndex.ToString();
        else if (control is DropDownList)
            cache[control.ID] = (control as DropDownList).SelectedIndex.ToString();
        else if (control is CheckBox)
            cache[control.ID] = (control as CheckBox).Checked.ToString();
        if (!(control is Repeater))
        {
            foreach (Control child in control.Controls)
                CacheControlValue(child, cache);
        }
    }
    #endregion TBI Instruments
}
