﻿using System;
using System.Collections.Generic;
using System.Data;
using System.Linq;
using System.Web.UI;
using System.Web.UI.WebControls;
using VeteransAffairs.Registries.Business;
using VeteransAffairs.Registries.BusinessManager;
using VeteransAffairs.Registries.BusinessManager.TBI;


public partial class TBI_Instruments_OptimalFollowUp : BaseTBIInstrumentPage
{

    #region Variable Declarations

    PatientManager patientManager = new PatientManager();
    InstrumentType instrumentType = new InstrumentType();

    protected int _surveyTypeId = 40; // Database Id for Optimal

    SURVEY _survey;   // the root object in the Survey set
    SURVEY_RESULT _surveyResult; // the object representing the actual data in the Survey

    protected List<string> _oneAnswers, _FiveDFiveAAnswers;

    List<STD_QUESTION> QuestionList = new List<STD_QUESTION>();

    #endregion


    /// <summary>
    /// Default Constructor
    /// </summary>
    public TBI_Instruments_OptimalFollowUp()
    {
        //Manager Save Notification Event Handler
        patientManager.BOSaveSuccess += manager_BOSaveSuccess;
    }

    #region Protected Methods

    /// <summary>
    /// Page Load Method
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    protected void Page_Load(object sender, EventArgs e)
    {
        TBIServiceTO _tbiServiceTo;

        try
        {
            // confirm instrument is active - if not, a redirect will occur.
            this.CheckActiveStatus(_surveyTypeId);

            _oneAnswers = new List<string>(10);
            _FiveDFiveAAnswers = new List<string>(8);

            _tbiServiceTo = (TBIServiceTO)Session["TBIServiceTO"];
            _tbiInstrumentManager.TbiServiceTo = _tbiServiceTo;

            string instTitle = _tbiInstrumentManager.GetInstrumentByID(_surveyTypeId).Name;

            LabelBreadCrumb.Text = "TBI Instruments > Instrument Associations > " + instTitle;
            LabelInstrumentTitle.Text = instTitle;
            Page.Title = "TBI Instruments - " + instTitle;

            if (_tbiInstrumentManager.TbiServiceTo != null)
            {
                Label labelUser = (Label)Master.FindControl("LabelCurrentUser");
                labelUser.Text = _tbiInstrumentManager.TbiServiceTo.UserName;

                LabelPatient.Text = _tbiInstrumentManager.TbiServiceTo.PatientName;
                LabelSSN.Text = _tbiInstrumentManager.TbiServiceTo.PatientSSN;
            }
        }
        catch (Exception ex)
        {
            //Redirect to message that session is ended and need to log back in from CPRS.
            Exception _tbilogerror = HandleTBIException(ex, "OptimalFollowUp.aspx", "100027", "SESSIOENDED");
            Response.Redirect("Warning.aspx?err=2", false);
        }

        if (!IsPostBack)
        {
            if (instrumentType.Status != null)
            {
                _tbiInstrumentManager.TbiServiceTo.SelectedInstrumentType = instrumentType;
            }

            string _ssn = "";
            if (_tbiInstrumentManager.TbiServiceTo != null)
            {
                _ssn = _tbiInstrumentManager.TbiServiceTo.PatientSSN;
            }

            if (!string.IsNullOrEmpty(_ssn))
            {
                string patICN = patientManager.GetPatientICNBySSN(_ssn);

                if (!string.IsNullOrEmpty(patICN))
                {
                    ViewState["PatientICN"] = patICN;
                    RefreshSurvey(patICN);
                }
                else
                {
                    throw new ArgumentException("Patient with this SSN does not exist.");
                }

                if (_survey != null && !_survey.isNew)
                {
                    SetUpSurvey(_survey);
                }
                string referrer = "";
                if (Request.UrlReferrer != null)
                {
                    referrer = Request.UrlReferrer.PathAndQuery;
                }
                ViewState["referrer"] = referrer;
            }

            //Check if the request is to start a new item 
            //a new item page request is signified when the querystring parameter id equals zero or is not present
            if (_tbiInstrumentManager.TbiServiceTo == null)
            {
                Logging.WriteLogToFile("Optimal.Page_Load(): WarningRedirect: _tbiInstrumentManager.TbiServiceTo == null");
                //Redirect to message that session is ended and need to log back in from CPRS.
                Response.Redirect("Warning.aspx?err=2");
            }
            else
            {
                LabelPatient.Text = _tbiInstrumentManager.TbiServiceTo.PatientName;
                string _SSN = _tbiInstrumentManager.TbiServiceTo.PatientSSN.Trim().Length == 9 ? String.Format("{0:000-00-0000}", int.Parse(_tbiInstrumentManager.TbiServiceTo.PatientSSN)) : _tbiInstrumentManager.TbiServiceTo.PatientSSN;
                //LabelFacility.Text = _tbiInstrumentManager.TbiServiceTo.InstitutionName;
                LabelSSN.Text = "*****" + _SSN.Substring(7, 4);
            }


        }

        QuestionList = GetAllQuestionsForSurvey();

        // Set control values for Diagnosis Question
        //ctrlDiagnosis.QuestionText = GetQuestionText("DX");
        //ctrlDiagnosis.SurveyTypeCode = HiddenSurveyType.Value;

        dpDate1.MaxDate = DateTime.Now;

    }

    protected override void OnPreRender(EventArgs e)
    {
        base.OnPreRender(e);
    }



    protected override void LoadViewState(object savedState)
    {
        base.LoadViewState(savedState);
        string surveyXml = (string)ViewState["surveyXml"];
        _survey = SerializerUtil.Deserialize<SURVEY>(surveyXml);

    }



    protected override object SaveViewState()
    {
        ViewState["surveyXml"] = _survey.Serialize();
        return base.SaveViewState();
    }

    /// <summary>
    /// Returns the Question_Text of the question with the passed Question_Number field
    /// </summary>
    /// <param name="questionId"></param>
    /// <returns></returns>
    protected string GetQuestionText(string questionId)
    {
        foreach (STD_QUESTION ques in QuestionList)
        {
            if (ques.QUESTION_NUMBER.ToLower() == questionId.ToLower())
            {
                return ques.QUESTION_TEXT;
            }
        }

        return "";
    }

    /// <summary>
    /// Returns the Question_Id of the question with the passed Question_Number field
    /// </summary>
    /// <param name="questionId"></param>
    /// <returns></returns>
    protected int GetQuestionId(string questionNumber)
    {
        foreach (STD_QUESTION ques in QuestionList)
        {
            if (ques.QUESTION_NUMBER.ToLower() == questionNumber.ToLower())
            {
                return ques.ID;
            }
        }

        return 0;
    }

    /// <summary>
    /// Save Draft button clicked.
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    public void btnSaveSurveyDraft_Click(object sender, EventArgs e)
    {
        Int32 status = 0;
        SaveFailMessage = "Save Failed. Please press " + btnSaveSurveyDraft.Text + " button again to avoid data loss.";
        SaveCompleteSurvey(ref status);
        SetUpSurvey(_survey);
    }

    /// <summary>
    /// Save and Prepare Note button clicked.
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    protected void btnSubmit_Click(object sender, EventArgs e)
    {
        Int32 status = 0;
        SaveFailMessage = "Save Failed. Please press " + btnSubmitSurvey.Text + " button again to avoid data loss.";
        SaveCompleteSurvey(ref status);

        if (status > 0)
        {
            Session["survey"] = _survey;
            Response.Redirect("InstrumentSummaryNote.aspx", false);
        }
        else
        {
            SetUpSurvey(_survey); //Follow Save Draft logic 
        }
    }

    /// <summary>
    /// Inactive timer firing.
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    protected void InActiveTimer_Tick(object sender, EventArgs e)
    {
        Int32 status = 0;
        SaveFailMessage = "Save Failed. Please press " + btnSaveSurveyDraft.Text + " button again to avoid data loss.";

        Page.Validate();

        if (Page.IsValid)
        {
            SaveCompleteSurvey(ref status);
            //SetUpSurvey(_survey);
            Response.Redirect("OptimalFollowUp.aspx", false);
        }
        else
        {
            SaveFailMessage = "Auto-Save Validation Failed. Please complete required fields in the survey and press " + btnSaveSurveyDraft.Text + " button again to avoid data loss.";
        }
    }

    /// <summary>
    /// Cancel button clicked.
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    protected void btnCancel_Click(object sender, EventArgs e)
    {
        ViewState["surveyXml"] = null;
        ViewState["PatientICN"] = null;
        ViewState["referrer"] = null;
        Session.Remove("survey");
        Response.Redirect("Instrument.aspx", false);
        //this.ClientScript.RegisterStartupScript(typeof(Page), "closePage", "<script type='text/javascript'> window.open('','_self','');window.close(); </script>");
    }

    /// <summary>
    /// Fired when scoring is updated.
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    protected void updateTotals(object sender, EventArgs e)
    {
        calculateTotals();
    }

    #endregion

    #region Private Methods

    /// <summary>
    /// Get all questions for this survey
    /// </summary>
    /// <returns></returns>
    private List<STD_QUESTION> GetAllQuestionsForSurvey()
    {
        return _tbiInstrumentManager.SelectAllQuestions(_surveyTypeId).ToList();
    }

    /// <summary>
    /// Refresh Survey
    /// </summary>
    /// <param name="patientICN"></param>
    private void RefreshSurvey(string patientICN)
    {
        var surveys = _tbiInstrumentManager.SelectByPatientICNAndSurveyType(patientICN, _surveyTypeId, true).ToList();

        if (surveys.Count > 0)
        {
            //SetUpSurvey(_survey);
            _survey = surveys.First();
        }
        else
        {
            _survey = new SURVEY();
        }

        _survey.SetAsChangeTrackingRoot();
    }

    /// <summary>
    /// Save Complete Survey
    /// </summary>
    /// <param name="status"></param>
    private void SaveCompleteSurvey(ref Int32 status)
    {
        Page.Validate();
        if (Page.IsValid)
        {

            calculateTotals();

            string patientICN = String.Empty;

            status = 0;

            //Retrieve patient's ICN from the session variable
            if (ViewState["PatientICN"] != null)
            {
                patientICN = ViewState["PatientICN"].ToString();
                if (String.IsNullOrEmpty(patientICN.Trim()))
                {
                    Logging.WriteLogToFile("Optimal.SaveCompletedSurvey(): WarningRedirect: String.IsNullOrEmpty(patientICN.Trim()");
                    //Lost state. Redirect to message that need to log back in from CPRS.
                    Response.Redirect("Warning.aspx?err=2", false);
                }
            }
            else
            {
                Logging.WriteLogToFile("Optimal.SaveCompletedSurvey(): WarningRedirect: ViewState[PatientICN] != null");
                //Lost state. Redirect to message that need to log back in from CPRS.
                Response.Redirect("Warning.aspx?err=2");
            }

            if (_survey != null)
            {
                if (_survey.isNew)
                {
                    //Ensure that a concurrent session hasn't already inserted a record for this survey
                    var surveys = _tbiInstrumentManager.SelectByPatientICNAndSurveyType(patientICN, _surveyTypeId, true).ToList();

                    if (surveys.Count > 0)
                    {
                        Logging.WriteLogToFile("Optimal.SaveCompletedSurvey(): WarningRedirect: surveys.Count > 0");
                        //A concurrent session has already inserted a record for this survey and it is not new anymore.
                        Response.Redirect("Warning.aspx?err=9", false);
                    }
                    else
                    {
                        _survey.SURVEY_STATUS = "inprocess";

                        _survey.SetAsInsertOnSubmit();

                        _survey.patient_ICN = patientICN;

                        _survey.STD_SURVEY_TYPE_ID = _surveyTypeId;

                        // TODO: change to datepicker
                        _survey.SURVEY_DATE = DateTime.Today;

                        //Save survey if it's new before adding any responses
                        SuppressSaveMessage = true;
                        _tbiInstrumentManager.Update(_survey);
                        SuppressSaveMessage = false;

                        RefreshSurvey(patientICN);
                    }
                }

                SaveSurvey(ref _survey);

                _survey.SetAsUpdateOnSubmit();
                status = _tbiInstrumentManager.Update(_survey);

                (this.Master as RegistryInstrumentMaster).SetDirtyFlag(false);

                RefreshSurvey(patientICN);
                SuppressSaveMessage = false;

            }
        }
    }

    /// <summary>
    /// Save survey.
    /// </summary>
    /// <param name="survey"></param>
    /// 
    private void SaveSurvey(ref SURVEY survey)
    {
        //List<STD_QUESTION> questions = GetAllQuestionsForSurvey();
        //Delete all the survey results from database if there are any.
        SURVEY surveyWithResults = _tbiInstrumentManager.GetSurveyWithAllAnswers(survey.SURVEYS_ID);
        _tbiInstrumentManager.DeleteAllSurveyResults(surveyWithResults);

        //Save pnlQuestions
        //SaveRadBtnListAnswers(pnlQuestions, survey);

        //_surveyResult = new SURVEY_RESULT();

        //try
        //{
            // Add Question DX choice
        //    _surveyResult.STD_QUESTION_CHOICE_ID = Convert.ToInt32(ctrlDiagnosis.GetSelectedId());
        //    _surveyResult.RESULT_TEXT = ctrlDiagnosis.GetSelectedResultText();
        //    survey.SURVEY_RESULTs.Add(_surveyResult);
        //}
        //catch (Exception surveyEx)
        //{
        //    Exception _tbilogerror = HandleTBIException(surveyEx, "OptimalFollowUp.aspx", "100027", "SaveSurvey() - Question DX");
        //}

        //Save pnlStatus
        //SaveRadBtnListAnswers(pnlStatus, survey);
        //SaveCheckListAnswers(pnlStatus, survey);

        //Save pnlBaselineDifficulty
        SaveRadBtnListAnswers(pnlBaselineDifficulty, survey);

        //Save pnlDropDownLists (have to do this because Calculate totals refreshing Q24)
        //SaveDropDownListAnswers(pnlDropDownLists, survey);


    }// END SaveSurvey
    private void setUpRadioButtons(Panel panel, SURVEY survey)
    {

        String _sControlID = "";


        foreach (var sResult in survey.SURVEY_RESULTs)
        {
            String s = sResult.STD_QUESTION_CHOICE.STD_QUESTION.QUESTION_NUMBER;
            if (s.Contains("Follow Up Difficulty-"))
            {
                _sControlID = s.Replace("Follow Up Difficulty-", "rblQdifficulty");
            }
            if (s.Contains("Follow Up Confidence-"))
            {
                _sControlID = s.Replace("Follow Up Confidence-", "rblQconfidence");
                //_sControlID = _sControlID + "c";
            }

            foreach (Control ctrl in panel.Controls)
            {
                if (ctrl is RadioButtonList)
                {
                    RadioButtonList rbl = (RadioButtonList)ctrl;
                    try
                    {
                        if (rbl.ID == _sControlID)
                        {
                            rbl.SelectedValue = sResult.STD_QUESTION_CHOICE.STD_QUESTION_CHOICE_ID.ToString();


                        }
                    }
                    catch (Exception surveyEx)
                    {
                        Exception _tbilogerror = HandleTBIException(surveyEx, "OptimalFollowUp.aspx", "100027", "setUpRadioButtons");
                    }
                }
            }
        }

    }

    /// <summary>
    /// Initialize/setup controls
    /// </summary>
    /// <param name="survey"></param>
    private void SetUpSurvey(SURVEY survey)
    {
        try
        {
            //check con first
            if (survey.SURVEY_RESULTs.Count > -1)
            {
                setUpRadioButtons(pnlBaselineDifficulty, survey);
            }
        }
        catch (Exception ex)
        {
            Exception _tbilogerror = HandleTBIException(ex, "OptimalFollowUp.aspx", "100027", "SetUpSurvey()");
        }
    }// END SetUpSurvey

    /// <summary>
    /// Retrieve instrument title
    /// </summary>
    /// <param name="surveyID"></param>
    /// <returns></returns>
    private string GetInstrumentTitle(int surveyID)
    {
        string ret = String.Empty;

        try
        {
            List<string> instrumentTitle = _tbiInstrumentManager.GetInstrumentByID(surveyID).Title;
            if (instrumentTitle.Count > 0)
            {
                foreach (string titleLine in instrumentTitle)
                {
                    if (titleLine.Contains("PhD"))
                    {
                        ret = ret + "<small>" + titleLine + "</small>" + "<br \\>";
                    }
                    else
                    {
                        ret = ret + titleLine + "<br \\>";
                    }
                }
            }

        }
        catch (Exception ex)
        {
            Exception _tbilogerror = HandleTBIException(ex, "OptimalFollowUp.aspx", "100027", "GetInstrumentTitle()");
            // Ignore error
        }

        return ret;
    }

    /// <summary>
    /// performs total calcuation
    /// </summary>
    private void calculateTotals()
    {
        #region calculate basic

        int basicScore = 0;

        foreach (Control ctrl in pnlBaselineDifficulty.Controls)
        {
            if (ctrl is RadioButtonList)
            {
                RadioButtonList rbl = (RadioButtonList)ctrl;

                for (int i = 0; i < ((RadioButtonList)ctrl).Items.Count; i++)
                {

                    if ((rbl.Items[i].Selected) && (i != 5))
                    {
                        try
                        {
                            basicScore += i + 1;
                        }
                        catch (Exception surveyEx)
                        {
                            Exception _tbilogerror = HandleTBIException(surveyEx, "OptimalFollowUp.aspx", "100027", "calculateTotals()- Baseline Difficulty");
                        }

                    }
                }
            }
        }

        lblBasicActivitiesScore.Text = basicScore.ToString();

        #endregion

    }

    #endregion


    //Replaces calls to GetChoicesByQuestionNumberandSurveyType
    private List<STD_QUESTION_CHOICE> GetQuestionChoices(string questionNumber)
    {
        return VeteransAffairs.Registries.QuestionChoiceRetriever.GetQuestionChoices(questionNumber);
    }


    private void SaveCheckAnswer(STD_QUESTION_CHOICE choice, List<string> answerList)
    {
        if (!IsNotAsked(choice.CHOICE_TEXT))
            answerList.Add(choice.STD_QUESTION_CHOICE_ID.ToString());
    }

    private void SaveDropDownListAnswers(Panel panel, SURVEY survey)
    {


        foreach (Control ctrl in panel.Controls)
        {
            if (ctrl is DropDownList)
            {
                DropDownList ddl = (DropDownList)ctrl;
                if (ddl.ID != "q24")
                {
                    for (int i = 0; i < ((DropDownList)ctrl).Items.Count; i++)
                    {
                        if (ddl.SelectedIndex == i)
                        {
                            _surveyResult = new SURVEY_RESULT();

                            try
                            {
                                // Add Question Living Arrangements
                                _surveyResult.STD_QUESTION_CHOICE_ID = Convert.ToInt32(ddl.SelectedItem.Value);
                                _surveyResult.RESULT_TEXT = ddl.SelectedItem.Text;
                                survey.SURVEY_RESULTs.Add(_surveyResult);
                            }
                            catch (Exception surveyEx)
                            {
                                Exception _tbilogerror = HandleTBIException(surveyEx, "OptimalFollowUp.aspx", "100027", "SaveSurvey() - SaveRadBtnListAnswers");
                            }

                        }
                    }
                }
            }
        }
    }


    private void SaveRadBtnListAnswers(Panel panel, SURVEY survey)
    {
        foreach (Control ctrl in panel.Controls)
        {
            if (ctrl is RadioButtonList)
            {
                RadioButtonList rbl = (RadioButtonList)ctrl;

                for (int i = 0; i < ((RadioButtonList)ctrl).Items.Count; i++)
                {
                    if (rbl.Items[i].Selected)
                    {
                        _surveyResult = new SURVEY_RESULT();

                        try
                        {
                            // Add Question Living Arrangements
                            _surveyResult.STD_QUESTION_CHOICE_ID = Convert.ToInt32(rbl.Items[i].Value);
                            _surveyResult.RESULT_TEXT = rbl.Items[i].Text;
                            survey.SURVEY_RESULTs.Add(_surveyResult);
                        }
                        catch (Exception surveyEx)
                        {
                            Exception _tbilogerror = HandleTBIException(surveyEx, "OptimalFollowUp.aspx", "100027", "SaveSurvey() - SaveRadBtnListAnswers");
                        }

                    }
                }
            }
        }
    }
    private void SaveCheckListAnswers(Panel panel, SURVEY survey)
    {
        foreach (Control ctrl in panel.Controls)
        {
            if (ctrl is CheckBoxList)
            {
                CheckBoxList chbx = (CheckBoxList)ctrl;

                for (int i = 0; i < ((CheckBoxList)ctrl).Items.Count; i++)
                {
                    if (chbx.Items[i].Selected)
                    {
                        _surveyResult = new SURVEY_RESULT();

                        try
                        {
                            // Add Question Living Arrangements
                            _surveyResult.STD_QUESTION_CHOICE_ID = Convert.ToInt32(chbx.Items[i].Value);
                            _surveyResult.RESULT_TEXT = chbx.Items[i].Text;
                            survey.SURVEY_RESULTs.Add(_surveyResult);
                        }
                        catch (Exception surveyEx)
                        {
                            Exception _tbilogerror = HandleTBIException(surveyEx, "OptimalFollowUp.aspx", "100027", "SaveSurvey() - SaveCheckListAnswers");
                        }

                    }
                }
            }
        }
    }
    private bool IsNotAsked(string s)
    {
        if (string.IsNullOrEmpty(s))
            return false;
        return s.IndexOf("not asked", StringComparison.OrdinalIgnoreCase) != -1;
    }

}
