﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.UI;
using System.Web.UI.WebControls;
using VeteransAffairs.Registries.Business;
using VeteransAffairs.Registries.BusinessManager;
using VeteransAffairs.Registries.BusinessManager.TBI;

public partial class TBI_Instruments_QUESTInstrument : BaseTBIInstrumentPage {

    #region Variable Declarations

    private const string FailedFormValuesName = "QUESTFailedFormValues";
    private const string DeferredMessageKeyName = "QUESTDeferredMessageKey";
    protected int _surveyTypeId = 13; // Database Id for QUEST

    PatientManager patientManager = new PatientManager();
    InstrumentType instrumentType = new InstrumentType();

    SURVEY _survey;   // the root object in the Survey set

    List<STD_QUESTION> QuestionList = new List<STD_QUESTION>();
    #endregion Variable Declarations

    #region Public Methods

    /// <summary>
    /// Default Constructor
    /// </summary>
    public TBI_Instruments_QUESTInstrument() {
        //Manager Save Notification Event Handler
        patientManager.BOSaveSuccess += manager_BOSaveSuccess;
    }
    #endregion Public Methods

    #region Protected Methods

    protected void Page_Init(object sender, EventArgs e)
    {
        VeteransAffairs.Registries.QuestionChoiceRetriever.Reset(_tbiInstrumentManager, _surveyTypeId);
    }

    protected void Page_Unload(object sender, EventArgs e)
    {
        VeteransAffairs.Registries.QuestionChoiceRetriever.Reset(null, -1);
    }

    bool preventNotification = false;
    protected void Page_PreRenderComplete(object sender, EventArgs e)
    {
        Dictionary<string, string> cache;
        if (!Page.IsPostBack && Helpers.SetupPageFromCache(this, FailedFormValuesName, out cache))
        {
            RestoreDateFromCache("D", cache);
            SuppressSaveMessage = false;
            SaveFailMessage = "Unable to save the survey.\r\n\r\nThere may be network issues preventing you from saving.\r\n\r\nPlease try to submit your changes again.";
            RenderStartUpMessage(false);
        }
        else if (Page.IsPostBack && !preventNotification)
        {
            DeferredMessage = "Save was Successful!";
        }
        else if (DeferredMessage != null)
        {
            SuppressSaveMessage = false;
            SaveSuccessMessage = DeferredMessage;
            RenderStartUpMessage(true);
            DeferredMessage = null;
        }
    }

    private void RestoreDateFromCache(string questionNumber, Dictionary<string, string> cache)
    {
        string dateString;
        if (cache.TryGetValue(questionNumber, out dateString))
        {
            dpDate1.SelectedDate = dpDate1.FocusedDate = DateTime.Parse(dateString);
        }
    }

    private static string DeferredMessage
    {
        get { return HttpContext.Current.Session[DeferredMessageKeyName] as string; }
        set { HttpContext.Current.Session[DeferredMessageKeyName] = value; }
    }

    /// <summary>
    /// Page Load method
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    protected void Page_Load(object sender, EventArgs e)
    {
        try
        {
            this.CheckActiveStatus(_surveyTypeId); // confirm instrument is active - if not, a redirect will occur.

            _tbiInstrumentManager.TbiServiceTo = base.GetTBIServiceTo();
            if (_tbiInstrumentManager.TbiServiceTo == null)
            {
                Logging.WriteLogToFile("QUESTInstrument.Page_Load(): _tbiInstrumentManager.TbiServiceTo == null");
                //Redirect to message that session is ended and need to log back in from CPRS.
                Response.Redirect("Warning.aspx?err=2");
            }
            string instTitle = _tbiInstrumentManager.GetInstrumentByID(_surveyTypeId).Name;

            LabelBreadCrumb.Text = "TBI Instruments > Instrument Associations > " + instTitle;
            LabelInstrumentTitle.Text = instTitle;
            Page.Title = "TBI Instruments - " + instTitle;

            Label labelUser = (Label)Master.FindControl("LabelCurrentUser");
            labelUser.Text = _tbiInstrumentManager.TbiServiceTo.UserName;

            LabelPatient.Text = _tbiInstrumentManager.TbiServiceTo.PatientName;
            LabelSSN.Text = _tbiInstrumentManager.TbiServiceTo.PatientSSN;
            if (!IsPostBack)
            {
                base.AddtoViewStateTBIServiceTo(_tbiInstrumentManager.TbiServiceTo);

                if (instrumentType.Status != null)
                {
                    _tbiInstrumentManager.TbiServiceTo.SelectedInstrumentType = instrumentType;
                }

                string _ssn = _tbiInstrumentManager.TbiServiceTo.PatientSSN;
                if (!string.IsNullOrEmpty(_ssn))
                {
                    string patICN = patientManager.GetPatientICNBySSN(_ssn);

                    if (!string.IsNullOrEmpty(patICN))
                    {
                        ViewState["PatientICN"] = patICN;
                        RefreshSurvey(patICN);
                    }
                    else
                    {
                        throw new ArgumentException("Patient with this SSN does not exist.");
                    }

                    if (_survey != null && !_survey.isNew)
                    {
                        SetUpSurvey(_survey);
                    }
                    string referrer = "";
                    if (Request.UrlReferrer != null)
                    {
                        referrer = Request.UrlReferrer.PathAndQuery;
                    }
                    ViewState["referrer"] = referrer;
                }
                LabelPatient.Text = _tbiInstrumentManager.TbiServiceTo.PatientName;
                LabelSSN.Text = _tbiInstrumentManager.TbiServiceTo.PatientSSN.Trim().Length == 9 ? String.Format("{0:000-00-0000}", int.Parse(_tbiInstrumentManager.TbiServiceTo.PatientSSN)) : _tbiInstrumentManager.TbiServiceTo.PatientSSN;
            }
            QuestionList = GetAllQuestionsForSurvey();
            SetQuestionLabels();

            dpDate1.MaxDate = DateTime.Now;
        }
        catch (Exception ex)
        {
            //Redirect to message that session is ended and need to log back in from CPRS.
            Exception _tbilogerror = HandleTBIException(ex, "QUESTInstrument.aspx", "100027", "Page_Load()");
            Response.Redirect("Warning.aspx?err=2", false);
        }
    }

    private void SetQuestionLabels()
    {
        LabelDate1.Text = GetQuestionText("D");

        // Set control values for Diagnosis Question
        ctrlDiagnosis.QuestionText = GetQuestionText("DX");
        ctrlDiagnosis.SurveyTypeCode = HiddenSurveyType.Value;

        // Set control values for Tech Dev Question
        LabelTechDev.Text = GetQuestionText("TechDev");

        // Set control values for questions
        LabelForQ1.Text = GetQuestionText("1");
        LabelFor1Comment.Text = GetQuestionText("1Comment");
        LabelForQ2.Text = GetQuestionText("2");
        LabelFor2Comment.Text = GetQuestionText("2Comment");
        LabelForQ3.Text = GetQuestionText("3");
        LabelFor3Comment.Text = GetQuestionText("3Comment");
        LabelForQ4.Text = GetQuestionText("4");
        LabelFor4Comment.Text = GetQuestionText("4Comment");
        LabelForQ5.Text = GetQuestionText("5");
        LabelFor5Comment.Text = GetQuestionText("5Comment");
        LabelForQ6.Text = GetQuestionText("6");
        LabelFor6Comment.Text = GetQuestionText("6Comment");
        LabelForQ7.Text = GetQuestionText("7");
        LabelFor7Comment.Text = GetQuestionText("7Comment");
        LabelForQ8.Text = GetQuestionText("8");
        LabelFor8Comment.Text = GetQuestionText("8Comment");
        LabelForQ9.Text = GetQuestionText("9");
        LabelFor9Comment.Text = GetQuestionText("9Comment");
        LabelForQ10.Text = GetQuestionText("10");
        LabelFor10Comment.Text = GetQuestionText("10Comment");
        LabelForQ11.Text = GetQuestionText("11");
        LabelFor11Comment.Text = GetQuestionText("11Comment");
        LabelForQ12.Text = GetQuestionText("12");
        LabelFor12Comment.Text = GetQuestionText("12Comment");
        LabelForQ13.Text = GetQuestionText("13");
    }

    protected override void LoadViewState(object savedState) {
        base.LoadViewState(savedState);
        string surveyXml = (string)ViewState["surveyXml"];
        _survey = SerializerUtil.Deserialize<SURVEY>(surveyXml);
    }

    protected override object SaveViewState() {
        ViewState["surveyXml"] = _survey.Serialize();
        return base.SaveViewState();
    }

    /// <summary>
    /// Returns the Question_Text of the question with the passed Question_Number field
    /// </summary>
    /// <param name="questionId"></param>
    /// <returns></returns>
    protected string GetQuestionText(string questionId)
    {
        foreach (STD_QUESTION ques in QuestionList)
        {
            if (ques.QUESTION_NUMBER.ToLower() == questionId.ToLower())
            {
                return ques.QUESTION_TEXT;
            }
        }
        return "";
    }

    /// <summary>
    /// Returns the Question_Id of the question with the passed Question_Number field
    /// </summary>
    /// <param name="questionId"></param>
    /// <returns></returns>
    protected int GetQuestionId(string questionNumber)
    {
        foreach (STD_QUESTION ques in QuestionList)
        {
            if (ques.QUESTION_NUMBER.ToLower() == questionNumber.ToLower())
            {
                return ques.ID;
            }
        }
        return 0;
    }

    /// <summary>
    /// Save Draft button clicked.
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    public void btnSaveSurveyDraft_Click(object sender, EventArgs e) {
        Int32 status = 0;
        SaveFailMessage = "Save Failed. Please press " + btnSaveSurveyDraft.Text + " button again to avoid data loss.";
        SaveCompleteSurvey(ref status);
        Response.Redirect(Request.Url.PathAndQuery, false);
    }

    /// <summary>
    /// Save and Prepare Note button clicked.
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    protected void btnSubmit_Click(object sender, EventArgs e)
    {
        Int32 status = 0;
        SaveFailMessage = "Save Failed. Please press " + btnSubmitSurvey.Text + " button again to avoid data loss.";
        lblImportantItemsError.Visible = false; //rehide and do validation
        Page.Validate();
        SaveCompleteSurvey(ref status);
        int importantCount = getImportantInfoCount();

        if (Page.IsValid && importantCount == 3 && status > 0)
        {
            preventNotification = true;
            Session["survey"] = _survey;
            Response.Redirect("InstrumentSummaryNote.aspx", false);
        }
        else if (Page.IsValid && importantCount != 3)
        {
            lblImportantItemsError.Visible = true;
            lblImportantItemsError.Text = "PLEASE SELECT THE THREE ITEMS that you consider to be the most important to you.";
        }
        else
        {
            lblImportantItemsError.Visible = true;
            lblImportantItemsError.Text = "PLEASE SELECT THE THREE ITEMS that you consider to be the most important to you.";
        }
    }

    /// <summary>
    /// Inactive timer firing.
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    protected void InActiveTimer_Tick(object sender, EventArgs e) {
        preventNotification = true;
        Int32 status = 0;
        SaveFailMessage = "Save Failed. Please press " + btnSaveSurveyDraft.Text + " button again to avoid data loss.";
        SaveCompleteSurvey(ref status);
        Response.Redirect(Request.Url.PathAndQuery, false);
    }

    /// <summary>
    /// Cancel button clicked.
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    protected void btnCancel_Click(object sender, EventArgs e) {
        ViewState["surveyXml"] = null;
        ViewState["PatientICN"] = null;
        ViewState["referrer"] = null;
        Session.Remove("survey");
        Response.Redirect("Instrument.aspx", false);
    }

    /// <summary>
    /// Call setup survey when important items are loaded.
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    protected void cblImportantItemsQ13_DataBound(object sender, EventArgs e) {
        SetUpSurvey(_survey);
    }

    #endregion

    #region Private Methods

    /// <summary>
    /// Get all questions for this survey
    /// </summary>
    /// <returns></returns>
    private List<STD_QUESTION> GetAllQuestionsForSurvey() {
        return _tbiInstrumentManager.SelectAllQuestions(_surveyTypeId).ToList();
    }

    /// <summary>
    /// Refresh Survey
    /// </summary>
    /// <param name="patientICN"></param>
    private void RefreshSurvey(string patientICN) {
        var surveys = _tbiInstrumentManager.SelectByPatientICNAndSurveyType(patientICN, _surveyTypeId, true).ToList();

        if (surveys.Count > 0) {
            _survey = surveys.First();
        } else {
            _survey = new SURVEY();
        }

        _survey.SetAsChangeTrackingRoot();
    }

    /// <summary>
    /// Save Complete Survey
    /// </summary>
    /// <param name="status"></param>
    private void SaveCompleteSurvey(ref Int32 status)
    {
        try
        {
            string patientICN = String.Empty;
            status = 0;
            //Retrieve patient's ICN from the session variable
            if (ViewState["PatientICN"] != null)
            {
                patientICN = ViewState["PatientICN"].ToString();
                if (String.IsNullOrEmpty(patientICN.Trim()))
                {
                    Logging.WriteLogToFile("QUESTInstrument.SaveCompleteSurvey(): WarningRedirect: String.IsNullOrEmpty(patientICN.Trim()");
                    //Lost state. Redirect to message that need to log back in from CPRS.
                    Response.Redirect("Warning.aspx?err=2", false);
                }
            }
            else
            {
                Logging.WriteLogToFile("QUESTInstrument.SaveCompleteSurvey(): WarningRedirect: ViewState[PatientICN] != null");
                //Lost state. Redirect to message that need to log back in from CPRS.
                Response.Redirect("Warning.aspx?err=2");
            }

            if (_survey != null)
            {
                if (_survey.isNew)
                {
                    //Ensure that a concurrent session hasn't already inserted a record for this survey
                    var surveys = _tbiInstrumentManager.SelectByPatientICNAndSurveyType(patientICN, _surveyTypeId, true).ToList();

                    if (surveys.Count > 0)
                    {
                        Logging.WriteLogToFile("QUESTInstrument.SaveCompleteSurvey(): WarningRedirect: surveys.Count > 0");
                        //A concurrent session has already inserted a record for this survey and it is not new anymore.
                        Response.Redirect("Warning.aspx?err=9", false);
                    }
                    else
                    {
                        _survey.SURVEY_STATUS = "inprocess";
                        _survey.SetAsInsertOnSubmit();
                        _survey.patient_ICN = patientICN;
                        _survey.STD_SURVEY_TYPE_ID = _surveyTypeId;

                        // TODO: change to datepicker
                        _survey.SURVEY_DATE = DateTime.Today;

                        //Save survey if it's new before adding any responses
                        SuppressSaveMessage = true;
                        _tbiInstrumentManager.Update(_survey);
                        SuppressSaveMessage = false;

                        RefreshSurvey(patientICN);
                    }
                }

                SaveSurvey(ref _survey);
                _survey.SetAsUpdateOnSubmit();
                status = _tbiInstrumentManager.Update(_survey);
                if (status != 0)
                {
                    //Fortify errors on this line (this.Master as RegistryInstrumentMaster).SetDirtyFlag(false);
                    ((RegistryInstrumentMaster)Master).SetDirtyFlag(false);
                    RefreshSurvey(patientICN);
                    SuppressSaveMessage = false;
                }
            }
        }
        catch (Exception ex)
        {
            Exception _tbilogerror = HandleTBIException(ex, "QUESTInstrument.aspx", "100027", "SaveCompleteSurvey()");
        }
        finally
        {
            if (status == 0)
            {
                var cache = Helpers.SavePageToCache(this, FailedFormValuesName);
                ctrlDiagnosis.WriteToCache(cache);
                SaveDateToCache(dpDate1, "D", cache);
            }
            else
                Session[FailedFormValuesName] = null;//clear out the cached version on success
        }
    }

    private static void SaveDateToCache(Telerik.Web.UI.RadDatePicker date, string questionNumber, Dictionary<string, string> cache)
    {
        if (date.SelectedDate != null)
        {
            cache.Add(questionNumber, date.SelectedDate.Value.ToShortDateString());
        }
    }

    private int SelectedDiagnosisID
    {
        get
        {
            string temp = this.ctrlDiagnosis.GetSelectedId();
            if (string.IsNullOrEmpty(temp))
                return 0;
            return Convert.ToInt32(temp);
        }
    }

    private void SaveCheckboxList(SURVEY survey, CheckBoxList checkBoxList, string questionNumber)
    {
        if (getImportantInfoCount() > 0)
        {
            string choiceStr = "";
            foreach (ListItem li in checkBoxList.Items)
            {
                if (li.Selected)
                {
                    choiceStr += li.Text + ",";
                }
            }

            if (!String.IsNullOrEmpty(choiceStr))
            {
                choiceStr = choiceStr.Substring(0, choiceStr.LastIndexOf(","));
                SaveText(survey, choiceStr, questionNumber);
            }
        }
    }

    private void SaveCustomDiagnosis(SURVEY survey, string questionNumber)
    {
        List<STD_QUESTION_CHOICE> availableChoices = VeteransAffairs.Registries.QuestionChoiceRetriever.GetQuestionChoices(questionNumber);
        SURVEY_RESULT currentResult = survey.SURVEY_RESULTs.FirstOrDefault(item => availableChoices.FirstOrDefault(choice => choice.STD_QUESTION_CHOICE_ID == item.STD_QUESTION_CHOICE_ID) != null);
        string inputText = ctrlDiagnosis.GetSelectedResultText();
        int selectedID = SelectedDiagnosisID;
        if (null != currentResult)
        {
            if (currentResult.STD_QUESTION_CHOICE_ID != selectedID || currentResult.RESULT_TEXT != inputText)
            {
                currentResult.SetAsUpdateOnSubmit();
                currentResult.RESULT_TEXT = inputText;
                currentResult.STD_QUESTION_CHOICE_ID = selectedID;
            }
            else
            {
                currentResult.SetAsNoChangeOnSubmit();
            }
        }
        else if (selectedID > 0)
        {
            SURVEY_RESULT result = new SURVEY_RESULT();
            result.RESULT_TEXT = inputText;
            result.STD_QUESTION_CHOICE_ID = selectedID;
            survey.SURVEY_RESULTs.Add(result);
        }
    }

    private void SaveDate(SURVEY survey, Telerik.Web.UI.RadDatePicker date, string questionNumber)
    {
        string inputText = "";
        if (date.SelectedDate != null)
        {
            inputText = ((DateTime)date.SelectedDate).ToShortDateString();
        }
        SaveText(survey, inputText, questionNumber);
    }

    private void SaveSingleSelect(SURVEY survey, RadioButtonList radioButtonList, string questionNumber)
    {
        if (radioButtonList.SelectedIndex > -1)
        {
            List<STD_QUESTION_CHOICE> availableChoices = VeteransAffairs.Registries.QuestionChoiceRetriever.GetQuestionChoices(questionNumber);
            SURVEY_RESULT currentResult = survey.SURVEY_RESULTs.FirstOrDefault(item => availableChoices.FirstOrDefault(choice => choice.STD_QUESTION_CHOICE_ID == item.STD_QUESTION_CHOICE_ID) != null);
            int selectedValue = Convert.ToInt32(radioButtonList.SelectedValue);
            if (null != currentResult)
            {
                if (currentResult.STD_QUESTION_CHOICE_ID != selectedValue || currentResult.RESULT_TEXT != radioButtonList.SelectedItem.Text)
                {
                    currentResult.SetAsUpdateOnSubmit();
                    currentResult.STD_QUESTION_CHOICE_ID = selectedValue;
                    currentResult.RESULT_TEXT = radioButtonList.SelectedItem.Text;
                }
                else
                {
                    currentResult.SetAsNoChangeOnSubmit();
                }
            }
            else
            {
                SURVEY_RESULT result = new SURVEY_RESULT();
                survey.SURVEY_RESULTs.Add(result);
                result.STD_QUESTION_CHOICE_ID = selectedValue;
                result.RESULT_TEXT = radioButtonList.SelectedItem.Text;
            }
        }
    }

    private void SaveText(SURVEY survey, string text, string questionNumber)
    {
        List<STD_QUESTION_CHOICE> availableChoices = VeteransAffairs.Registries.QuestionChoiceRetriever.GetQuestionChoices(questionNumber);
        SURVEY_RESULT currentResult = survey.SURVEY_RESULTs.FirstOrDefault(item => availableChoices.FirstOrDefault(choice => choice.STD_QUESTION_CHOICE_ID == item.STD_QUESTION_CHOICE_ID) != null);
        if (null != currentResult)
        {
            if (currentResult.RESULT_TEXT != text)
            {
                currentResult.SetAsUpdateOnSubmit();
                currentResult.RESULT_TEXT = text;
            }
            else
            {
                currentResult.SetAsNoChangeOnSubmit();
            }
        }
        else if (text != string.Empty)
        {
            SURVEY_RESULT result = new SURVEY_RESULT();
            result.RESULT_TEXT = text;
            result.STD_QUESTION_CHOICE_ID = availableChoices.FirstOrDefault().STD_QUESTION_CHOICE_ID;
            survey.SURVEY_RESULTs.Add(result);
        }
    }

    /// <summary>
    /// Save survey.
    /// </summary>
    /// <param name="survey"></param>
    private void SaveSurvey(ref SURVEY survey) {
        string questionNumber = "";

        try
        {
            //first, mark all current answers for deletion
            survey.SetAsChangeTrackingRoot();
            foreach (SURVEY_RESULT tempResult in survey.SURVEY_RESULTs)
                tempResult.SetAsDeleteOnSubmit();

            SaveCustomDiagnosis(survey, questionNumber = "DX");
            SaveDate(survey, dpDate1, questionNumber = "D");
            SaveText(survey, txtTechDev.Text, questionNumber = "TechDev");
            SaveSingleSelect(survey, rblQ1, questionNumber = "1");
            SaveText(survey, txtComments1.Text, questionNumber = "1Comment");
            SaveSingleSelect(survey, rblQ2, questionNumber = "2");
            SaveText(survey, txtComments2.Text, questionNumber = "2Comment");
            SaveSingleSelect(survey, rblQ3, questionNumber = "3");
            SaveText(survey, txtComments3.Text, questionNumber = "3Comment");
            SaveSingleSelect(survey, rblQ4, questionNumber = "4");
            SaveText(survey, txtComments4.Text, questionNumber = "4Comment");
            SaveSingleSelect(survey, rblQ5, questionNumber = "5");
            SaveText(survey, txtComments5.Text, questionNumber = "5Comment");
            SaveSingleSelect(survey, rblQ6, questionNumber = "6");
            SaveText(survey, txtComments6.Text, questionNumber = "6Comment");
            SaveSingleSelect(survey, rblQ7, questionNumber = "7");
            SaveText(survey, txtComments7.Text, questionNumber = "7Comment");
            SaveSingleSelect(survey, rblQ8, questionNumber = "8");
            SaveText(survey, txtComments8.Text, questionNumber = "8Comment");
            SaveSingleSelect(survey, rblQ9, questionNumber = "9");
            SaveText(survey, txtComments9.Text, questionNumber = "9Comment");
            SaveSingleSelect(survey, rblQ10, questionNumber = "10");
            SaveText(survey, txtComments10.Text, questionNumber = "10Comment");
            SaveSingleSelect(survey, rblQ11, questionNumber = "11");
            SaveText(survey, txtComments11.Text, questionNumber = "11Comment");
            SaveSingleSelect(survey, rblQ12, questionNumber = "12");
            SaveText(survey, txtComments12.Text, questionNumber = "12Comment");
            SaveCheckboxList(survey, cblImportantItemsQ13, questionNumber = "13");
            SaveText(survey, "0", questionNumber = "14");  // Question 14 - always 0 - required
            SaveText(survey, calculateDeviceSubscaleScore().ToString(), questionNumber = "15");
            SaveText(survey, calculateServicesSubscaleScore().ToString(), questionNumber = "16");
            SaveText(survey, calculateTotalQUESTScore().ToString(), questionNumber = "17");
        }
        catch (Exception surveyEx)
        {
            Exception _tbilogerror = HandleTBIException(surveyEx, "QUESTInstrument.aspx", "100027", "SaveSurvey() - Question " + questionNumber);
            throw;
        }
    }// END SaveSurvey

    private void SetCheckboxList(SURVEY_RESULT result, CheckBoxList checkBoxList)
    {
        if (result.STD_QUESTION_CHOICE.STD_QUESTION_CHOICE_ID > -1)
        {
            // get result text ... split into array
            string[] importantItemArray = result.RESULT_TEXT.Split(',');
            foreach (string s in importantItemArray)
            {
                foreach (ListItem li in checkBoxList.Items)
                {
                    if (s.ToLower() == li.Text.ToLower())
                    {
                        li.Selected = true;
                        break;
                    }
                }
            }
        }
    }

    private void SetCustomDiagnosisElement(SURVEY_RESULT result, ASP.controls_tbidiagnosisselector_ascx diagControl)
    {
        if (result.STD_QUESTION_CHOICE.STD_QUESTION_CHOICE_ID > -1)
        {
            diagControl.SelectedChoiceId = result.STD_QUESTION_CHOICE.STD_QUESTION_CHOICE_ID.ToString();
            if (result.STD_QUESTION_CHOICE.CHOICE_TEXT.ToLower().Contains("other"))
            {
                diagControl.SetOtherDescriptionText(result.RESULT_TEXT);
            }
        }
        else
        {
            diagControl.SelectedChoiceId = null;
        }
    }

    private void SetDateElement(SURVEY_RESULT result, Telerik.Web.UI.RadDatePicker date)
    {
        if (!String.IsNullOrEmpty(result.RESULT_TEXT))
        {
            DateTime temp;
            if (DateTime.TryParse(result.RESULT_TEXT, out temp))
            {
                date.SelectedDate = temp;
            }
        }
    }

    private void SetRadioElement(SURVEY_RESULT result, RadioButtonList radioButton)
    {
        if (result.STD_QUESTION_CHOICE.STD_QUESTION_CHOICE_ID > -1)
        {
            radioButton.SelectedValue = result.STD_QUESTION_CHOICE.STD_QUESTION_CHOICE_ID.ToString();
        }
        else
        {
            radioButton.SelectedIndex = -1;
        }
    }

    private void SetTextBoxElement(SURVEY_RESULT result, TextBox textBox)
    {
        if (!String.IsNullOrEmpty(result.RESULT_TEXT))
        {
            textBox.Text = result.RESULT_TEXT;
        }
    }

    /// <summary>
    /// Initialize/setup controls
    /// </summary>
    /// <param name="survey"></param>
    private void SetUpSurvey(SURVEY survey)
    {
        try
        {
            foreach (var sResult in survey.SURVEY_RESULTs)
            {
                switch (sResult.STD_QUESTION_CHOICE.STD_QUESTION.QUESTION_NUMBER)
                {
                    case "DX": SetCustomDiagnosisElement(sResult, ctrlDiagnosis); break;
                    case "D": SetDateElement(sResult, dpDate1); break;
                    case "TechDev": SetTextBoxElement(sResult, txtTechDev); break;
                    case "1": SetRadioElement(sResult, rblQ1); break;
                    case "1Comment": SetTextBoxElement(sResult, txtComments1); break;
                    case "2": SetRadioElement(sResult, rblQ2); break;
                    case "2Comment": SetTextBoxElement(sResult, txtComments2); break;
                    case "3": SetRadioElement(sResult, rblQ3); break;
                    case "3Comment": SetTextBoxElement(sResult, txtComments3); break;
                    case "4": SetRadioElement(sResult, rblQ4); break;
                    case "4Comment": SetTextBoxElement(sResult, txtComments4); break;
                    case "5": SetRadioElement(sResult, rblQ5); break;
                    case "5Comment": SetTextBoxElement(sResult, txtComments5); break;
                    case "6": SetRadioElement(sResult, rblQ6); break;
                    case "6Comment": SetTextBoxElement(sResult, txtComments6); break;
                    case "7": SetRadioElement(sResult, rblQ7); break;
                    case "7Comment": SetTextBoxElement(sResult, txtComments7); break;
                    case "8": SetRadioElement(sResult, rblQ8); break;
                    case "8Comment": SetTextBoxElement(sResult, txtComments8); break;
                    case "9": SetRadioElement(sResult, rblQ9); break;
                    case "9Comment": SetTextBoxElement(sResult, txtComments9); break;
                    case "10": SetRadioElement(sResult, rblQ10); break;
                    case "10Comment": SetTextBoxElement(sResult, txtComments10); break;
                    case "11": SetRadioElement(sResult, rblQ11); break;
                    case "11Comment": SetTextBoxElement(sResult, txtComments11); break;
                    case "12": SetRadioElement(sResult, rblQ12); break;
                    case "12Comment": SetTextBoxElement(sResult, txtComments12); break;
                    case "13": SetCheckboxList(sResult, cblImportantItemsQ13); break;    
                    default: break;
                }
            }
        }
        catch (Exception ex)
        {
            Exception _tbilogerror = HandleTBIException(ex, "QUESTInstrument.aspx", "100027", "SetUpSurvey()");
        }
    }// END SetUpSurvey

    /// <summary>
    /// Retrieve instrument title
    /// </summary>
    /// <param name="surveyID"></param>
    /// <returns></returns>
    private string GetInstrumentTitle(int surveyID) {
        string ret = String.Empty;

        try {
            List<string> instrumentTitle = _tbiInstrumentManager.GetInstrumentByID(surveyID).Title;
            if (instrumentTitle.Count > 0) {
                foreach (string titleLine in instrumentTitle) {
                    if (titleLine.Contains("PhD")) {
                        ret = ret + "<small>" + titleLine + "</small>" + "<br \\>";
                    } else {
                        ret = ret + titleLine + "<br \\>";
                    }
                }
            }

        } catch (Exception ex) {
            Exception _tbilogerror = HandleTBIException(ex, "QUESTInstrument.aspx", "100027", "GetInstrumentTitle()");
            // Ignore error
        }
        return ret;
    }

    /// <summary>
    /// Retrieves count of selected items in cblImportantItemsQ13
    /// </summary>
    /// <returns></returns>
    private int getImportantInfoCount()
    {
        int selectedItems = 0;

        foreach (ListItem li in cblImportantItemsQ13.Items)
        {
            if (li.Selected)
            {
                selectedItems++;
            }
        }
        return selectedItems;
    }

    /// <summary>
    /// For items 1 to 8, add the ratings of the valid responses and divide this sum by the number of valid items in this scale
    /// </summary>
    /// <returns></returns>
    private decimal calculateDeviceSubscaleScore() {
        decimal subScaleScore = 0;

        try {
            // look at rbl value field (convert to int)
            int q1 = Convert.ToInt32(rblQ1.SelectedItem.Text);
            int q2 = Convert.ToInt32(rblQ2.SelectedItem.Text);
            int q3 = Convert.ToInt32(rblQ3.SelectedItem.Text);
            int q4 = Convert.ToInt32(rblQ4.SelectedItem.Text);
            int q5 = Convert.ToInt32(rblQ5.SelectedItem.Text);
            int q6 = Convert.ToInt32(rblQ6.SelectedItem.Text);
            int q7 = Convert.ToInt32(rblQ7.SelectedItem.Text);
            int q8 = Convert.ToInt32(rblQ8.SelectedItem.Text);

            subScaleScore = (Convert.ToDecimal(q1 + q2 + q3 + q4 + q5 + q6 + q7 + q8) / Convert.ToDecimal(8));
            subScaleScore = Math.Round(subScaleScore, 2);

        } catch (Exception ex) {
            Exception _tbilogerror = HandleTBIException(ex, "QUESTInstrument.aspx", "100027", "calculateDeviceSubscaleScore()");
        }

        return subScaleScore;
    }

    /// <summary>
    /// For items 9 to 12, add the ratings of the valid responses and divide this sum by the number of valid items in this scale.
    /// </summary>
    /// <returns></returns>
    private decimal calculateServicesSubscaleScore() {
        decimal subScaleScore = 0;

        try {
            // look at rbl value field (convert to int)
            int q9 = Convert.ToInt32(rblQ9.SelectedItem.Text);
            int q10 = Convert.ToInt32(rblQ10.SelectedItem.Text);
            int q11 = Convert.ToInt32(rblQ11.SelectedItem.Text);
            int q12 = Convert.ToInt32(rblQ12.SelectedItem.Text);

            subScaleScore = (Convert.ToDecimal(q9 + q10 + q11 + q12) / Convert.ToDecimal(4));
            subScaleScore = Math.Round(subScaleScore, 2);

        } catch (Exception ex) {
            Exception _tbilogerror = HandleTBIException(ex, "QUESTInstrument.aspx", "100027", "calculateServicesSubscaleScore()");
        }
        return subScaleScore;
    }

    /// <summary>
    /// For items 1 to 12, add the ratings of the valid responses and divide this sum by the number of valid items
    /// </summary>
    /// <returns></returns>
    private decimal calculateTotalQUESTScore() {
        decimal totalScore = 0;

        try {
            // look at rbl value field (convert to int)
            int q1 = Convert.ToInt32(rblQ1.SelectedItem.Text);
            int q2 = Convert.ToInt32(rblQ2.SelectedItem.Text);
            int q3 = Convert.ToInt32(rblQ3.SelectedItem.Text);
            int q4 = Convert.ToInt32(rblQ4.SelectedItem.Text);
            int q5 = Convert.ToInt32(rblQ5.SelectedItem.Text);
            int q6 = Convert.ToInt32(rblQ6.SelectedItem.Text);
            int q7 = Convert.ToInt32(rblQ7.SelectedItem.Text);
            int q8 = Convert.ToInt32(rblQ8.SelectedItem.Text);
            int q9 = Convert.ToInt32(rblQ9.SelectedItem.Text);
            int q10 = Convert.ToInt32(rblQ10.SelectedItem.Text);
            int q11 = Convert.ToInt32(rblQ11.SelectedItem.Text);
            int q12 = Convert.ToInt32(rblQ12.SelectedItem.Text);

            totalScore = (Convert.ToDecimal(q1 + q2 + q3 + q4 + q5 + q6 + q7 + q8 + q9 + q10 + q11 + q12) / Convert.ToDecimal(12));
            totalScore = Math.Round(totalScore, 2);

        } catch (Exception ex) {
            Exception _tbilogerror = HandleTBIException(ex, "QUESTInstrument.aspx", "100027", "calculateTotalQUESTScore()");
        }
        return totalScore;
    }
    #endregion
}